@@ -15,7 +15,8 @@ format.sfc = function(x, ..., width = 30) {
 
 #' Create simple feature geometry list column
 #'
-#' Create simple feature geometry list column, set class, and add coordinate reference system and precision
+#' Create simple feature geometry list column, set class, and add coordinate reference system and precision.
+#' For data.frame alternatives see [st_sf()]. To convert a foreign object to `sfc`, see [st_as_sfc()]
 #'
 #' @name sfc
 #' @aliases sfc_POINT sfc_LINESTRING sfc_POLYGON sfc_MULTIPOINT sfc_MULTILINESTRING sfc_MULTIPOLYGON sfc_GEOMETRYCOLLECTION
@@ -35,6 +36,7 @@ format.sfc = function(x, ..., width = 30) {
 #' pt1 = st_point(c(0,1))
 #' pt2 = st_point(c(1,1))
 #' (sfc = st_sfc(pt1, pt2))
+#' sfc[sfc[1], op = st_is_within_distance, dist = 0.5]
 #' d = st_sf(data.frame(a=1:2, geom=sfc))
 #' @export
 st_sfc = function(..., crs = NA_crs_, precision = 0.0, check_ring_dir = FALSE, dim,
@@ -43,6 +45,8 @@ st_sfc = function(..., crs = NA_crs_, precision = 0.0, check_ring_dir = FALSE, d
 	# if we have only one arg, which is already a list with sfg's, but NOT a geometrycollection:
 	# (this is the old form of calling st_sfc; it is way faster to call st_sfc(lst) if lst
 	# already contains a zillion sfg objects, than do.call(st_sfc, lst) ...
+	if (length(lst) && inherits(lst[[1]], "sf"))
+		stop("use st_as_sfc() to extract geometries from an sf object")
 	if (length(lst) == 1 && is.list(lst[[1]]) && !inherits(lst[[1]], "sfg")
 			&& (length(lst[[1]]) == 0 || inherits(lst[[1]][[1]], "sfg") || is.null(lst[[1]][[1]])))
 		lst = lst[[1]]
@@ -96,12 +100,12 @@ st_sfc = function(..., crs = NA_crs_, precision = 0.0, check_ring_dir = FALSE, d
 
 	# compute bbox, if not set:
 	bb = attr(lst, "bbox")
-	if (is.null(bb) || any(is.na(bb)) || recompute_bbox)
+	if (is.null(bb) || anyNA(bb) || recompute_bbox)
 		attr(lst, "bbox") = compute_bbox(lst)
 
 	# compute z_range, if dims permit and not set
 	zr = attr(lst, "z_range")
-	if (is.null(zr) || any(is.na(zr))) {
+	if (is.null(zr) || anyNA(zr)) {
 		u <- dims_and_types[[1]]
 		if( "XYZM" %in% u ) {
 			attr(lst, "z_range") = compute_z_range(lst)
@@ -136,12 +140,22 @@ st_sfc = function(..., crs = NA_crs_, precision = 0.0, check_ring_dir = FALSE, d
 	lst
 }
 
+#' @name sfc
+#' @param x object of class \code{sfc}
+#' @param i record selection. Might also be an \code{sfc}/\code{sf} object to work with the \code{op} argument
+#' @param j ignored
+#' @param op function, geometrical binary predicate function to apply when
+#'   \code{i} is a \code{sf}/\code{sfc} object. Additional arguments can be
+#'   specified using \code{...}, see examples.
 #' @export
 "[.sfc" = function(x, i, j, ..., op = st_intersects) {
 	if (!missing(i) && (inherits(i, "sf") || inherits(i, "sfc") || inherits(i, "sfg")))
 		i = lengths(op(x, i, ...)) != 0
-	st_sfc(NextMethod(), crs = st_crs(x), precision = st_precision(x),
-		dim = if(length(x)) class(x[[1]])[1] else "XY")
+	precision = st_precision(x)
+	crs = st_crs(x)
+	dim = if (length(x)) class(x[[1]])[1] else "XY"
+	x = unclass(x)[i] # now a list
+	st_sfc(x, crs = crs, precision = precision, dim = dim)
 }
 
 
@@ -321,9 +335,10 @@ st_geometry_type = function(x, by_geometry = TRUE) {
 #' Drop Z and/or M dimensions from feature geometries, resetting classes appropriately
 #' @param x object of class \code{sfg}, \code{sfc} or \code{sf}
 #' @param ... ignored
-#' @param drop logical; drop, or (FALSE) add?
+#' @param drop logical; drop, or (`FALSE`) add?
 #' @param what character which dimensions to drop or add
-#' @details Only combinations \code{drop=TRUE}, \code{what = "ZM"}, and \code{drop=FALSE}, \code{what="Z"} are supported so far. In case \code{add=TRUE}, \code{x} should have \code{XY} geometry, and zero values are added for \code{Z}.
+#' @details Only combinations \code{drop=TRUE}, \code{what = "ZM"}, and \code{drop=FALSE}, \code{what="Z"} are supported so far.
+#' In case \code{add=TRUE}, \code{x} should have \code{XY} geometry, and zero values are added for \code{Z}.
 #' @examples
 #' st_zm(st_linestring(matrix(1:32,8)))
 #' x = st_sfc(st_linestring(matrix(1:32,8)), st_linestring(matrix(1:8,2)))
@@ -403,11 +418,14 @@ st_precision.sfc <- function(x) {
 
 #' Set precision
 #'
-#' @name st_precision
+#' @rdname st_precision
 #' @param precision numeric, or object of class \code{units} with distance units (but see details); see \link{st_as_binary} for how to do this.
 #' @details If \code{precision} is a \code{units} object, the object on which we set precision must have a coordinate reference system with compatible distance units.
 #'
-#' Setting a \code{precision} has no direct effect on coordinates of geometries, but merely set an attribute tag to an \code{sfc} object. The effect takes place in \link{st_as_binary} or, more precise, in the C++ function \code{CPL_write_wkb}, where simple feature geometries are being serialized to well-known-binary (WKB). This happens always when routines are called in GEOS library (geometrical operations or predicates), for writing geometries using \link{st_write} or \link{write_sf}, \code{st_make_valid} in package \code{lwgeom}; also \link{aggregate} and \link{summarise} by default union geometries, which calls a GEOS library function. Routines in these libraries receive rounded coordinates, and possibly return results based on them. \link{st_as_binary} contains an example of a roundtrip of \code{sfc} geometries through WKB, in order to see the rounding happening to R data.
+#' Setting a \code{precision} has no direct effect on coordinates of geometries, but merely set an attribute tag to an \code{sfc} object.
+#' The effect takes place in \link{st_as_binary} or, more precise, in the C++ function \code{CPL_write_wkb}, where simple feature geometries are being serialized to well-known-binary (WKB).
+#' This happens always when routines are called in GEOS library (geometrical operations or predicates), for writing geometries using \link{st_write} or \link{write_sf}, \code{st_make_valid} in package \code{lwgeom}; also \link{aggregate} and \link{summarise} by default union geometries, which calls a GEOS library function.
+#' Routines in these libraries receive rounded coordinates, and possibly return results based on them. \link{st_as_binary} contains an example of a roundtrip of \code{sfc} geometries through WKB, in order to see the rounding happening to R data.
 #'
 #' The reason to support precision is that geometrical operations in GEOS or liblwgeom may work better at reduced precision. For writing data from R to external resources it is harder to think of a good reason to limiting precision.
 #'
