@@ -835,6 +835,13 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 	} else if (op == "linemerge") {
 		for (size_t i = 0; i < g.size(); i++)
 			out[i] = geos_ptr(chkNULL(GEOSLineMerge_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
+	} else if (op == "linemergedirected") {
+#ifdef HAVE311
+		for (size_t i = 0; i < g.size(); i++)
+			out[i] = geos_ptr(chkNULL(GEOSLineMergeDirected_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
+#else
+		Rcpp::stop("directed line merge requires GEOS >= 3.11");
+#endif
 	} else if (op == "polygonize") {
 		for (size_t i = 0; i < g.size(); i++) {
 			const GEOSGeometry* gi = g[i].get();
@@ -943,7 +950,9 @@ Rcpp::List CPL_geos_op2(std::string op, Rcpp::List sfcx, Rcpp::List sfcy) {
 	std::vector<GeomPtr> out;
 	std::vector<double> index_x, index_y;
 	std::vector<size_t> items(x.size());
+#ifdef HAVE_390
 	double grid_size = geos_grid_size_xy(sfcx, sfcy);
+#endif
 
 	if (op == "intersection") {
 
@@ -1185,7 +1194,9 @@ Rcpp::List CPL_nary_difference(Rcpp::List sfc) {
 	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
 	std::vector<GeomPtr> x = geometries_from_sfc(hGEOSCtxt, sfc, &dim);
 	std::vector<GeomPtr> out;
+#ifdef HAVE_390
 	double grid_size = geos_grid_size(sfc);
+#endif
 	// initialize trees to find overlapping areas quickly
 	for (size_t i = 0; i < x.size(); i++) {
 		// if i'th geometry in x is empty then skip it
@@ -1255,7 +1266,9 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 	std::vector<GeomPtr> x = geometries_from_sfc(hGEOSCtxt, sfc, &dim);
 	std::vector<GeomPtr> out;
 	int errors = 0;
+#ifdef HAVE_390
 	double grid_size = geos_grid_size(sfc);
+#endif
 #ifdef HAVE350
 	notice = 0;
 	GEOSContext_setNoticeMessageHandler_r(hGEOSCtxt,
@@ -1363,3 +1376,41 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 	// return result
 	return ret;
 }
+
+// [[Rcpp::export]]
+Rcpp::NumericVector CPL_line_project(Rcpp::List lines, Rcpp::List points, bool normalized) {
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
+	int dim = 2;
+	std::vector<GeomPtr> l = geometries_from_sfc(hGEOSCtxt, lines, &dim);
+	std::vector<GeomPtr> p = geometries_from_sfc(hGEOSCtxt, points, &dim);
+	Rcpp::NumericVector ret(p.size());
+	if (normalized) {
+		for (size_t i = 0; i < l.size() && i < p.size(); i++)
+			ret[i] = GEOSProjectNormalized_r(hGEOSCtxt, l[i].get(), p[i].get());
+	} else {
+		for (size_t i = 0; i < l.size() && i < p.size(); i++)
+			ret[i] = GEOSProject_r(hGEOSCtxt, l[i].get(), p[i].get());
+	}
+	CPL_geos_finish(hGEOSCtxt);
+	// return result
+	return ret;
+}
+
+// [[Rcpp::export]]
+Rcpp::List CPL_line_interpolate(Rcpp::List lines, Rcpp::NumericVector dists, bool normalized) {
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
+	int dim = 2;
+	std::vector<GeomPtr> l = geometries_from_sfc(hGEOSCtxt, lines, &dim);
+	std::vector<GeomPtr> p(l.size());
+	if (normalized) {
+		for (int i = 0; i < (int) l.size() && i < dists.size(); i++)
+			p[i] = geos_ptr(GEOSInterpolateNormalized_r(hGEOSCtxt, l[i].get(), dists[i]), hGEOSCtxt);
+	} else {
+		for (int i = 0; i < (int) l.size() && i < dists.size(); i++)
+			p[i] = geos_ptr(GEOSInterpolate_r(hGEOSCtxt, l[i].get(), dists[i]), hGEOSCtxt);
+	}
+	Rcpp::List ret(sfc_from_geometry(hGEOSCtxt, p, dim));
+	CPL_geos_finish(hGEOSCtxt);
+	// return result
+	return ret;
+}
